@@ -15,33 +15,40 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "animation.h"
-#include "animation_action.h"
-#include "animation_cycle.h"
-#include "bone.h"
-#include "coreanimation.h"
-#include "corebone.h"
-#include "corekeyframe.h"
-#include "corematerial.h"
-#include "coremesh.h"
-#include "coremodel.h"
-#include "coreskeleton.h"
-#include "coresubmesh.h"
-#include "coretrack.h"
-#include "error.h"
-#include "loader.h"
-#include "matrix.h"
-#include "mesh.h"
-#include "mixer.h"
-#include "model.h"
-#include "physique.h"
-#include "quaternion.h"
-#include "renderer.h"
-#include "saver.h"
-#include "skeleton.h"
-#include "springsystem.h"
-#include "submesh.h"
-#include "vector.h"
+#include "cal3d/animation.h"
+#include "cal3d/animation_action.h"
+#include "cal3d/animation_cycle.h"
+#include "cal3d/bone.h"
+#include "cal3d/coreanimation.h"
+#include "cal3d/corebone.h"
+#include "cal3d/corekeyframe.h"
+#include "cal3d/corematerial.h"
+#include "cal3d/coremesh.h"
+#include "cal3d/coremodel.h"
+#include "cal3d/coremorphanimation.h"
+#include "cal3d/coreskeleton.h"
+#include "cal3d/coresubmesh.h"
+#include "cal3d/coresubmorphtarget.h"
+#include "cal3d/coretrack.h"
+#include "cal3d/datasource.h"
+#include "cal3d/error.h"
+#include "cal3d/hardwaremodel.h"
+#include "cal3d/loader.h"
+#include "cal3d/matrix.h"
+#include "cal3d/mesh.h"
+#include "cal3d/mixer.h"
+#include "cal3d/model.h"
+#include "cal3d/morphtargetmixer.h"
+#include "cal3d/physique.h"
+#include "cal3d/platform.h"
+#include "cal3d/quaternion.h"
+#include "cal3d/renderer.h"
+#include "cal3d/saver.h"
+#include "cal3d/skeleton.h"
+#include "cal3d/springsystem.h"
+#include "cal3d/streamsource.h"
+#include "cal3d/submesh.h"
+#include "cal3d/vector.h"
 
 #endif
 
