@@ -11,46 +11,30 @@
 #ifndef CAL_COREKEYFRAME_H
 #define CAL_COREKEYFRAME_H
 
-//****************************************************************************//
-// Includes                                                                   //
-//****************************************************************************//
 
-#include "global.h"
-#include "matrix.h"
-#include "vector.h"
-#include "quaternion.h"
+#include "cal3d/global.h"
+#include "cal3d/matrix.h"
+#include "cal3d/vector.h"
+#include "cal3d/quaternion.h"
 
-//****************************************************************************//
-// Class declaration                                                          //
-//****************************************************************************//
-
- /*****************************************************************************/
-/** The core keyframe class.
-  *****************************************************************************/
 
 class CAL3D_API CalCoreKeyframe
 {
-// member variables
-protected:
-  float m_time;
-  CalVector m_translation;
-  CalQuaternion m_rotation;
-
-// constructors/destructor
 public:
   CalCoreKeyframe();
-  virtual ~CalCoreKeyframe();
+  ~CalCoreKeyframe() { }
 
-// member functions
-public:
-  bool create();
-  void destroy();
   const CalQuaternion& getRotation();
   float getTime();
   const CalVector& getTranslation();
   void setRotation(const CalQuaternion& rotation);
   void setTime(float time);
   void setTranslation(const CalVector& translation);
+
+private:
+  float m_time;
+  CalVector m_translation;
+  CalQuaternion m_rotation;
 };
 
 #endif
