@@ -16,19 +16,19 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "error.h"
-#include "mixer.h"
-#include "coremodel.h"
-#include "corebone.h"
-#include "coreanimation.h"
-#include "coretrack.h"
-#include "corekeyframe.h"
-#include "model.h"
-#include "skeleton.h"
-#include "bone.h"
-#include "animation.h"
-#include "animation_action.h"
-#include "animation_cycle.h"
+#include "cal3d/error.h"
+#include "cal3d/mixer.h"
+#include "cal3d/coremodel.h"
+#include "cal3d/corebone.h"
+#include "cal3d/coreanimation.h"
+#include "cal3d/coretrack.h"
+#include "cal3d/corekeyframe.h"
+#include "cal3d/model.h"
+#include "cal3d/skeleton.h"
+#include "cal3d/bone.h"
+#include "cal3d/animation.h"
+#include "cal3d/animation_action.h"
+#include "cal3d/animation_cycle.h"
 
  /*****************************************************************************/
 /** Constructs the mixer instance.
@@ -36,8 +36,23 @@
   * This function is the default constructor of the mixer instance.
   *****************************************************************************/
 
-CalMixer::CalMixer()
+CalMixer::CalMixer(CalModel* pModel)
 {
+  assert(pModel);
+
+  m_pModel = pModel;
+
+  // build the animation table
+  int coreAnimationCount = m_pModel->getCoreModel()->getCoreAnimationCount();
+
+  m_vectorAnimation.reserve(coreAnimationCount);
+  CalAnimation* null = 0;
+  m_vectorAnimation.insert(m_vectorAnimation.begin(), coreAnimationCount, null);
+
+  // set the animation time/duration values to default
+  m_animationTime = 0.0f;
+  m_animationDuration = 0.0f;
+  m_timeFactor = 1.0f;
 }
 
  /*****************************************************************************/
@@ -48,9 +63,29 @@ CalMixer::CalMixer()
 
 CalMixer::~CalMixer()
 {
-  assert(m_vectorAnimation.empty());
-  assert(m_listAnimationCycle.empty());
-  assert(m_listAnimationAction.empty());
+  // destroy all active animation actions
+  while(!m_listAnimationAction.empty())
+  {
+    CalAnimationAction *pAnimationAction = m_listAnimationAction.front();
+    m_listAnimationAction.pop_front();
+
+    delete pAnimationAction;
+  }
+
+  // destroy all active animation cycles
+  while(!m_listAnimationCycle.empty())
+  {
+    CalAnimationCycle *pAnimationCycle;
+    pAnimationCycle = m_listAnimationCycle.front();
+    m_listAnimationCycle.pop_front();
+
+    delete pAnimationCycle;
+  }
+
+  // clear the animation table
+  m_vectorAnimation.clear();
+
+  m_pModel = 0;
 }
 
  /*****************************************************************************/
@@ -78,8 +113,7 @@ bool CalMixer::blendCycle(int id, float weight, float delay)
   }
 
   // get the animation for the given id
-  CalAnimation *pAnimation;
-  pAnimation = m_vectorAnimation[id];
+  CalAnimation *pAnimation = m_vectorAnimation[id];
 
   // create a new animation instance if it is not active yet
   if(pAnimation == 0)
@@ -88,50 +122,46 @@ bool CalMixer::blendCycle(int id, float weight, float delay)
     if(weight == 0.0f) return true;
 
     // get the core animation
-    CalCoreAnimation *pCoreAnimation;
-    pCoreAnimation = m_pModel->getCoreModel()->getCoreAnimation(id);
+    CalCoreAnimation *pCoreAnimation = m_pModel->getCoreModel()->getCoreAnimation(id);
     if(pCoreAnimation == 0) return false;
 
-	CalCoreTrack *coreTrack = pCoreAnimation->getCoreTrack(0);
-		
+	std::list<CalCoreTrack*>& listCoreTrack = pCoreAnimation->getListCoreTrack();
+
+    if(listCoreTrack.size() == 0) return false;
+    
+	CalCoreTrack *coreTrack = listCoreTrack.front();
+    if(coreTrack == 0) return false;
+
 	CalCoreKeyframe *lastKeyframe = coreTrack->getCoreKeyframe(coreTrack->getCoreKeyframeCount()-1);
-	
+    if(lastKeyframe == 0) return false;
+
 	if(lastKeyframe->getTime()<pCoreAnimation->getDuration())
 	{
-		std::list<CalCoreTrack*>& listCoreTrack = pCoreAnimation->getListCoreTrack();
 
-		std::list<CalCoreTrack *>::iterator itr;		
+		std::list<CalCoreTrack *>::iterator itr;
 		for(itr=listCoreTrack.begin();itr!=listCoreTrack.end();++itr)
 		{
 			coreTrack = *itr;
-			
-            CalCoreKeyframe *firstKeyframe = coreTrack->getCoreKeyframe(0);			
+
+            CalCoreKeyframe *firstKeyframe = coreTrack->getCoreKeyframe(0);
 			CalCoreKeyframe *newKeyframe = new CalCoreKeyframe();
-            
+
 			newKeyframe->setTranslation(firstKeyframe->getTranslation());
             newKeyframe->setRotation(firstKeyframe->getRotation());
             newKeyframe->setTime(pCoreAnimation->getDuration());
 
 			coreTrack->addCoreKeyframe(newKeyframe);
-		}	
+		}
 	}
 
     // allocate a new animation cycle instance
-    CalAnimationCycle *pAnimationCycle;
-    pAnimationCycle = new CalAnimationCycle();
+    CalAnimationCycle *pAnimationCycle = new CalAnimationCycle(pCoreAnimation);
     if(pAnimationCycle == 0)
     {
       CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
       return false;
     }
 
-    // create the new animation instance
-    if(!pAnimationCycle->create(pCoreAnimation))
-    {
-      delete pAnimationCycle;
-      return false;
-    }
-
     // insert new animation into the tables
     m_vectorAnimation[id] = pAnimationCycle;
     m_listAnimationCycle.push_front(pAnimationCycle);
@@ -158,7 +188,9 @@ bool CalMixer::blendCycle(int id, float weight, float delay)
   pAnimationCycle = (CalAnimationCycle *)pAnimation;
 
   // blend the animation cycle
-  return pAnimationCycle->blend(weight, delay);
+  pAnimationCycle->blend(weight, delay);
+  pAnimationCycle->checkCallbacks(0,m_pModel);
+  return true;
 }
 
  /*****************************************************************************/
@@ -209,115 +241,31 @@ bool CalMixer::clearCycle(int id, float delay)
   pAnimationCycle->setAsync(m_animationTime, m_animationDuration);
 
   // blend the animation cycle
-  return pAnimationCycle->blend(0.0f, delay);
-}
-
- /*****************************************************************************/
-/** Creates the mixer instance.
-  *
-  * This function creates the mixer instance.
-  *
-  * @param pModel A pointer to the model that should be managed with this mixer
-  *               instance.
-  *
-  * @return One of the following values:
-  *         \li \b true if successful
-  *         \li \b false if an error happend
-  *****************************************************************************/
-
-bool CalMixer::create(CalModel *pModel)
-{
-  if(pModel == 0)
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
-  m_pModel = pModel;
-
-  CalCoreModel *pCoreModel;
-  pCoreModel = m_pModel->getCoreModel();
-  if(pCoreModel == 0)
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
-  // build the animation table
-  int coreAnimationCount;
-  coreAnimationCount = m_pModel->getCoreModel()->getCoreAnimationCount();
-
-  m_vectorAnimation.reserve(coreAnimationCount);
-  m_vectorAnimation.insert(m_vectorAnimation.begin(), (std::vector<CalAnimation *>::size_type)coreAnimationCount, 0);
-
-  // set the animation time/duration values to default
-  m_animationTime = 0.0f;
-  m_animationDuration = 0.0f;
-
+  pAnimationCycle->blend(0.0f, delay);
+  pAnimationCycle->checkCallbacks(0, m_pModel);
   return true;
 }
 
- /*****************************************************************************/
-/** Destroys the mixer instance.
-  *
-  * This function destroys all data stored in the mixer instance and frees all
-  * allocated memory.
-  *****************************************************************************/
-
-void CalMixer::destroy()
-{
-  // destroy all active animation actions
-  while(!m_listAnimationAction.empty())
-  {
-    CalAnimationAction *pAnimationAction;
-    pAnimationAction = m_listAnimationAction.front();
-    m_listAnimationAction.pop_front();
-
-    pAnimationAction->destroy();
-    delete pAnimationAction;
-  }
-
-  // destroy all active animation cycles
-  while(!m_listAnimationCycle.empty())
-  {
-    CalAnimationCycle *pAnimationCycle;
-    pAnimationCycle = m_listAnimationCycle.front();
-    m_listAnimationCycle.pop_front();
-
-    pAnimationCycle->destroy();
-    delete pAnimationCycle;
-  }
-
-  // clear the animation table
-  m_vectorAnimation.clear();
-
-  m_pModel = 0;
-}
-
- /*****************************************************************************/
+/*****************************************************************************/
 /** Executes an animation action.
   *
   * This function executes an animation action.
   *
-  * @param id The ID of the animation cycle that should be blended.
+  * @param id The ID of the animation action that should be blended.
   * @param delayIn The time in seconds until the animation action reaches the
   *                full weight from the beginning of its execution.
   * @param delayOut The time in seconds in which the animation action reaches
   *                 zero weight at the end of its execution.
+  * @param weightTarget The weight to interpolate the animation action to.
+  * @param autoLock     This prevents the Action from being reset and removed
+  *                     on the last keyframe if true.
   *
   * @return One of the following values:
   *         \li \b true if successful
   *         \li \b false if an error happend
   *****************************************************************************/
-
-bool CalMixer::executeAction(int id, float delayIn, float delayOut)
+bool CalMixer::executeAction(int id, float delayIn, float delayOut, float weightTarget, bool autoLock)
 {
-  if((id < 0) || (id >= (int)m_vectorAnimation.size()))
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
   // get the core animation
   CalCoreAnimation *pCoreAnimation;
   pCoreAnimation = m_pModel->getCoreModel()->getCoreAnimation(id);
@@ -327,26 +275,62 @@ bool CalMixer::executeAction(int id, float delayIn, float delayOut)
   }
 
   // allocate a new animation action instance
-  CalAnimationAction *pAnimationAction;
-  pAnimationAction = new CalAnimationAction();
+  CalAnimationAction *pAnimationAction = new CalAnimationAction(pCoreAnimation);
   if(pAnimationAction == 0)
   {
     CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
     return false;
   }
 
-  // create the new animation instance
-  if(!pAnimationAction->create(pCoreAnimation))
-  {
-    delete pAnimationAction;
-    return false;
-  }
-
   // insert new animation into the table
   m_listAnimationAction.push_front(pAnimationAction);
 
   // execute the animation
-  return pAnimationAction->execute(delayIn, delayOut);
+  pAnimationAction->execute(delayIn, delayOut, weightTarget, autoLock);
+  pAnimationAction->checkCallbacks(0, m_pModel);
+  return true;
+}
+
+/*****************************************************************************/
+/** Clears an active animation action.
+  *
+  * This function removes an animation action from the blend list.  This is
+  * particularly useful with auto-locked actions on their last frame.
+  *
+  * @param id The ID of the animation action that should be removed.
+  *
+  * @return One of the following values:
+  *         \li \b true if successful
+  *         \li \b false if an error happened or action was not found
+  *****************************************************************************/
+bool CalMixer::removeAction(int id)
+{
+  // get the core animation
+  CalCoreAnimation *pCoreAnimation;
+  pCoreAnimation = m_pModel->getCoreModel()->getCoreAnimation(id);
+  if(pCoreAnimation == 0)
+  {
+    return false;
+  }
+
+  // update all active animation actions of this model
+  std::list<CalAnimationAction *>::iterator iteratorAnimationAction;
+  iteratorAnimationAction = m_listAnimationAction.begin();
+
+  while(iteratorAnimationAction != m_listAnimationAction.end())
+  {
+    // find the specified action and remove it
+    if((*iteratorAnimationAction)->getCoreAnimation() == pCoreAnimation )
+    {
+        // found, so remove
+      (*iteratorAnimationAction)->completeCallbacks(m_pModel);
+      delete (*iteratorAnimationAction);
+      iteratorAnimationAction = m_listAnimationAction.erase(iteratorAnimationAction);
+      return true;
+    }
+    iteratorAnimationAction++;
+  }
+  return false;
 }
 
  /*****************************************************************************/
@@ -367,11 +351,14 @@ void CalMixer::updateAnimation(float deltaTime)
   }
   else
   {
-    m_animationTime += deltaTime;
+    m_animationTime += deltaTime * m_timeFactor;
     if(m_animationTime >= m_animationDuration)
     {
       m_animationTime = (float) fmod(m_animationTime, m_animationDuration);
     }
+	if (m_animationTime < 0)
+      m_animationTime += m_animationDuration;
+
   }
 
   // update all active animation actions of this model
@@ -383,12 +370,13 @@ void CalMixer::updateAnimation(float deltaTime)
     // update and check if animation action is still active
     if((*iteratorAnimationAction)->update(deltaTime))
     {
+      (*iteratorAnimationAction)->checkCallbacks(m_animationTime,m_pModel);
       ++iteratorAnimationAction;
     }
     else
     {
       // animation action has ended, destroy and remove it from the animation list
-      (*iteratorAnimationAction)->destroy();
+      (*iteratorAnimationAction)->completeCallbacks(m_pModel);
       delete (*iteratorAnimationAction);
       iteratorAnimationAction = m_listAnimationAction.erase(iteratorAnimationAction);
     }
@@ -416,12 +404,13 @@ void CalMixer::updateAnimation(float deltaTime)
         accumulatedDuration += (*iteratorAnimationCycle)->getWeight() * (*iteratorAnimationCycle)->getCoreAnimation()->getDuration();
       }
 
+      (*iteratorAnimationCycle)->checkCallbacks(m_animationTime,m_pModel);
       ++iteratorAnimationCycle;
     }
     else
     {
       // animation cycle has ended, destroy and remove it from the animation list
-      (*iteratorAnimationCycle)->destroy();
+      (*iteratorAnimationCycle)->completeCallbacks(m_pModel);
       delete (*iteratorAnimationCycle);
       iteratorAnimationCycle = m_listAnimationCycle.erase(iteratorAnimationCycle);
     }
@@ -438,12 +427,6 @@ void CalMixer::updateAnimation(float deltaTime)
   }
 }
 
- /*****************************************************************************/
-/** Updates the skeleton.
-  *
-  * This function updates the skeleton to match the current animation state.
-  *****************************************************************************/
-
 void CalMixer::updateSkeleton()
 {
   // get the skeleton we need to update
@@ -543,4 +526,72 @@ void CalMixer::updateSkeleton()
   pSkeleton->calculateState();
 }
 
+/*****************************************************************************/
+/** Returns the animation time.
+  *
+  * This function returns the animation time of the mixer instance.
+  *
+  * @return The animation time in seconds.
+  *****************************************************************************/
+
+
+float CalMixer::getAnimationTime()
+{
+	return m_animationTime;
+}
+
+/*****************************************************************************/
+/** Returns the animation duration.
+  *
+  * This function returns the animation duration of the mixer instance.
+  *
+  * @return The animation duration in seconds.
+  *****************************************************************************/
+
+
+float CalMixer::getAnimationDuration()
+{
+	return m_animationDuration;
+}
+
+
+/*****************************************************************************/
+/** Sets the animation time.
+  *
+  * This function sets the animation time of the mixer instance.
+  *
+  *****************************************************************************/
+
+
+void CalMixer::setAnimationTime(float animationTime)
+{
+	m_animationTime=animationTime;
+}
+
+/*****************************************************************************/
+/** Set the time factor.
+  * 
+  * This function sets the time factor of the mixer instance.
+  * this time factor affect only sync animation
+  *
+  *****************************************************************************/
+
+
+void CalMixer::setTimeFactor(float timeFactor)
+{
+    m_timeFactor = timeFactor;
+}
+
+/*****************************************************************************/
+/** Get the time factor.
+  * 
+  * This function return the time factor of the mixer instance.
+  *
+  *****************************************************************************/
+
+float CalMixer::getTimeFactor()
+{
+    return m_timeFactor;
+}
+
 //****************************************************************************//
