@@ -16,19 +16,20 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "error.h"
-#include "model.h"
-#include "skeleton.h"
-#include "bone.h"
-#include "mixer.h"
-#include "renderer.h"
-#include "coremodel.h"
-#include "coreskeleton.h"
-#include "coremesh.h"
-#include "coresubmesh.h"
-#include "mesh.h"
-#include "physique.h"
-#include "springsystem.h"
+#include "cal3d/error.h"
+#include "cal3d/model.h"
+#include "cal3d/skeleton.h"
+#include "cal3d/bone.h"
+#include "cal3d/mixer.h"
+#include "cal3d/renderer.h"
+#include "cal3d/coremodel.h"
+#include "cal3d/coreskeleton.h"
+#include "cal3d/coremesh.h"
+#include "cal3d/coresubmesh.h"
+#include "cal3d/mesh.h"
+#include "cal3d/morphtargetmixer.h"
+#include "cal3d/physique.h"
+#include "cal3d/springsystem.h"
 
  /*****************************************************************************/
 /** Constructs the model instance.
@@ -36,14 +37,26 @@
   * This function is the default constructor of the model instance.
   *****************************************************************************/
 
-CalModel::CalModel()
+CalModel::CalModel(CalCoreModel* pCoreModel)
+  : m_pCoreModel(0)
+  , m_pSkeleton(0)
+  , m_pMixer(0)
+  , m_pMorphTargetMixer(0)
+  , m_pPhysique(0)
+  , m_pSpringSystem(0)
+  , m_pRenderer(0)
+  , m_userData(0)
 {
-  m_pCoreModel = 0;
-  m_pSkeleton = 0;
-  m_pMixer = 0;
-  m_pPhysique = 0;
-  m_pSpringSystem = 0;
-  m_pRenderer = 0;
+  assert(pCoreModel);
+
+  m_pCoreModel = pCoreModel;
+  m_pSkeleton = new CalSkeleton(pCoreModel->getCoreSkeleton());
+  m_pMixer = new CalMixer(this);
+  m_pMorphTargetMixer = new CalMorphTargetMixer(this);
+  m_pPhysique = new CalPhysique(this);
+  m_pSpringSystem = new CalSpringSystem(this);
+  m_pRenderer = new CalRenderer(this);
+
   m_userData = 0;
 }
 
@@ -55,7 +68,17 @@ CalModel::CalModel()
 
 CalModel::~CalModel()
 {
-  assert(m_vectorMesh.empty());
+  for(size_t meshId = 0; meshId < m_vectorMesh.size(); ++meshId)
+  {
+    delete m_vectorMesh[meshId];
+  }
+
+  delete m_pRenderer;
+  delete m_pSpringSystem;
+  delete m_pPhysique;
+  delete m_pMixer;
+  delete m_pMorphTargetMixer;
+  delete m_pSkeleton;
 }
 
  /*****************************************************************************/
@@ -96,21 +119,13 @@ bool CalModel::attachMesh(int coreMeshId)
   }
 
   // allocate a new mesh instance
-  CalMesh *pMesh;
-  pMesh = new CalMesh();
+  CalMesh *pMesh = new CalMesh(pCoreMesh);
   if(pMesh == 0)
   {
     CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
     return false;
   }
 
-  // create the new mesh instance
-  if(!pMesh->create(pCoreMesh))
-  {
-    delete pMesh;
-    return false;
-  }
-
   // set model in the mesh instance
   pMesh->setModel(this);
 
@@ -120,225 +135,6 @@ bool CalModel::attachMesh(int coreMeshId)
   return true;
 }
 
- /*****************************************************************************/
-/** Creates the model instance.
-  *
-  * This function creates the model instance based on a core model.
-  *
-  * @param pCoreModel A pointer to the core model on which this model instance
-  *                   should be based on.
-  *
-  * @return One of the following values:
-  *         \li \b true if successful
-  *         \li \b false if an error happend
-  *****************************************************************************/
-
-bool CalModel::create(CalCoreModel *pCoreModel)
-{
-  if(pCoreModel == 0)
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
-  m_pCoreModel = pCoreModel;
-
-  // allocate a new skeleton instance
-  CalSkeleton *pSkeleton;
-  pSkeleton = new CalSkeleton();
-  if(pSkeleton == 0)
-  {
-    CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
-    return false;
-  }
-
-  // create the skeleton from the core skeleton
-  if(!pSkeleton->create(pCoreModel->getCoreSkeleton()))
-  {
-    delete pSkeleton;
-    return false;
-  }
-
-  m_pSkeleton = pSkeleton;
-
-  // allocate a new mixer instance
-  CalMixer *pMixer;
-  pMixer = new CalMixer();
-  if(pMixer == 0)
-  {
-    CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    return false;
-  }
-
-  // create the mixer from this model
-  if(!pMixer->create(this))
-  {
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    delete pMixer;
-    return false;
-  }
-
-  m_pMixer = pMixer;
-
-  // allocate a new physqiue instance
-  CalPhysique *pPhysique;
-  pPhysique = new CalPhysique();
-  if(pPhysique == 0)
-  {
-    CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    return false;
-  }
-
-  // create the physique from this model
-  if(!pPhysique->create(this))
-  {
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    delete pPhysique;
-    return false;
-  }
-
-  m_pPhysique = pPhysique;
-
-  // allocate a new spring system instance
-  CalSpringSystem *pSpringSystem;
-  pSpringSystem = new CalSpringSystem();
-  if(pSpringSystem == 0)
-  {
-    CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
-    m_pPhysique->destroy();
-    delete m_pPhysique;
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    return false;
-  }
-
-  // create the spring system from this model
-  if(!pSpringSystem->create(this))
-  {
-    m_pPhysique->destroy();
-    delete m_pPhysique;
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    delete pSpringSystem;
-    return false;
-  }
-
-  m_pSpringSystem = pSpringSystem;
-
-  // allocate a new renderer instance
-  CalRenderer *pRenderer;
-  pRenderer = new CalRenderer();
-  if(pRenderer == 0)
-  {
-    CalError::setLastError(CalError::MEMORY_ALLOCATION_FAILED, __FILE__, __LINE__);
-    m_pSpringSystem->destroy();
-    delete m_pSpringSystem;
-    m_pPhysique->destroy();
-    delete m_pPhysique;
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    return false;
-  }
-
-  // create the renderer from this model
-  if(!pRenderer->create(this))
-  {
-    m_pSpringSystem->destroy();
-    delete m_pSpringSystem;
-    m_pPhysique->destroy();
-    delete m_pPhysique;
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    delete pRenderer;
-    return false;
-  }
-
-  m_pRenderer = pRenderer;
-
-  // initialize the user data
-  m_userData = 0;
-
-  return true;
-}
-
- /*****************************************************************************/
-/** Destroys the model instance.
-  *
-  * This function destroys all data stored in the model instance and frees all
-  * allocated memory.
-  *****************************************************************************/
-
-void CalModel::destroy()
-{
-  // destroy all active meshes
-  int meshId;
-  for(meshId = 0; meshId < (int)m_vectorMesh.size(); ++meshId)
-  {
-    m_vectorMesh[meshId]->destroy();
-    delete m_vectorMesh[meshId];
-  }
-  m_vectorMesh.clear();
-
-  // destroy the renderer instance
-  if(m_pRenderer != 0)
-  {
-    m_pRenderer->destroy();
-    delete m_pRenderer;
-    m_pRenderer = 0;
-  }
-
-  // destroy the spring system instance
-  if(m_pSpringSystem != 0)
-  {
-    m_pSpringSystem->destroy();
-    delete m_pSpringSystem;
-    m_pSpringSystem = 0;
-  }
-
-  // destroy the physique instance
-  if(m_pPhysique != 0)
-  {
-    m_pPhysique->destroy();
-    delete m_pPhysique;
-    m_pPhysique = 0;
-  }
-
-  // destroy the mixer instance
-  if(m_pMixer != 0)
-  {
-    m_pMixer->destroy();
-    delete m_pMixer;
-    m_pMixer = 0;
-  }
-
-  // destroy the skeleton instance
-  if(m_pSkeleton != 0)
-  {
-    m_pSkeleton->destroy();
-    delete m_pSkeleton;
-    m_pSkeleton = 0;
-  }
-
-  m_pCoreModel = 0;
-}
 
  /*****************************************************************************/
 /** Detaches a mesh.
@@ -377,7 +173,6 @@ bool CalModel::detachMesh(int coreMeshId)
     if(pMesh->getCoreMesh() == pCoreMesh)
     {
       // destroy the mesh
-      pMesh->destroy();
       delete pMesh;
 
       // erase the mesh out of the active mesh list
@@ -401,7 +196,7 @@ bool CalModel::detachMesh(int coreMeshId)
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalCoreModel *CalModel::getCoreModel()
+CalCoreModel *CalModel::getCoreModel() const
 {
   return m_pCoreModel;
 }
@@ -418,7 +213,7 @@ CalCoreModel *CalModel::getCoreModel()
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalMesh *CalModel::getMesh(int coreMeshId)
+CalMesh *CalModel::getMesh(int coreMeshId) const
 {
   // check if the id is valid
   if((coreMeshId < 0) ||(coreMeshId >= m_pCoreModel->getCoreMeshCount()))
@@ -428,12 +223,10 @@ CalMesh *CalModel::getMesh(int coreMeshId)
   }
 
   // get the core mesh
-  CalCoreMesh *pCoreMesh;
-  pCoreMesh = m_pCoreModel->getCoreMesh(coreMeshId);
+  CalCoreMesh *pCoreMesh = m_pCoreModel->getCoreMesh(coreMeshId);
 
   // search the mesh
-  int meshId;
-  for(meshId = 0; meshId < (int)m_vectorMesh.size(); ++meshId)
+  for(int meshId = 0; meshId < (int)m_vectorMesh.size(); ++meshId)
   {
     // check if we found the matching mesh
     if(m_vectorMesh[meshId]->getCoreMesh() == pCoreMesh)
@@ -445,19 +238,83 @@ CalMesh *CalModel::getMesh(int coreMeshId)
   return 0;
 }
 
- /*****************************************************************************/
-/** Provides access to the mixer.
+/*****************************************************************************/
+/** Returns the mixer.
+ *
+ * If a mixer that is not an instance of CalMixer was set with the
+ * CalModel::setAbstractMixer method, an INVALID_MIXER_TYPE error (see
+ * CalError) is set and 0 is returned.
+ *
+ * @return \li a pointer to the mixer
+ *         \li \b 0 if an error happend
+ *****************************************************************************/
+
+CalMixer *CalModel::getMixer() const
+{
+  if(m_pMixer == 0)
+    return 0;
+
+  if(m_pMixer->isDefaultMixer() == false) {
+    CalError::setLastError(CalError::INVALID_MIXER_TYPE, __FILE__, __LINE__);
+    return 0;
+  } else {
+    return (CalMixer*)(m_pMixer);
+  }
+}
+
+/*****************************************************************************/
+/** Returns the mixer. 
+ *
+ * @return \li a pointer to the mixer
+ *         \li \b 0 if no mixer was set
+ *****************************************************************************/
+
+CalAbstractMixer *CalModel::getAbstractMixer() const
+{
+  return m_pMixer;
+}
+
+/*****************************************************************************/
+/** Sets the mixer to a CalAbstractMixer subclass instance.
+ *
+ * If a mixer was already set (with CalModel::setAbstractMixer or
+ * because the CalModel::create method created a CalMixer instance),
+ * its \b destroy method is called. The existing mixer is not
+ * deallocated, it is the responsibility of the caller to call the
+ * getAbstractMixer method and deallocate the returned instance if
+ * appropriate.
+ *
+ * \b pMixer will be deallocated by cal3d if and only if the
+ * CalModel::destroy function is called.
+ *
+ * The \b create method of pMixer is called.
+ *
+ * pMixer may be null. After setting a null pointer, the caller MUST
+ * call CalModel::create or CalModel::setAbstractMixer with a non-null
+ * pointer before any other method is called.
+ *
+ * @param pMixer is a pointer to a CalAbstractMixer subclass instance.
+ *
+ *****************************************************************************/
+
+void CalModel::setAbstractMixer(CalAbstractMixer* pMixer)
+{
+  m_pMixer = pMixer;
+}
+
+/*****************************************************************************/
+/** Provides access to the morph target mixer.
   *
-  * This function returns the mixer.
+  * This function returns the morph target mixer.
   *
   * @return One of the following values:
-  *         \li a pointer to the mixer
+  *         \li a pointer to the morph target mixer
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalMixer *CalModel::getMixer()
+CalMorphTargetMixer *CalModel::getMorphTargetMixer() const
 {
-  return m_pMixer;
+  return m_pMorphTargetMixer;
 }
 
  /*****************************************************************************/
@@ -470,7 +327,7 @@ CalMixer *CalModel::getMixer()
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalPhysique *CalModel::getPhysique()
+CalPhysique *CalModel::getPhysique() const
 {
   return m_pPhysique;
 }
@@ -485,7 +342,7 @@ CalPhysique *CalModel::getPhysique()
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalRenderer *CalModel::getRenderer()
+CalRenderer *CalModel::getRenderer() const
 {
   return m_pRenderer;
 }
@@ -500,7 +357,7 @@ CalRenderer *CalModel::getRenderer()
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalSkeleton *CalModel::getSkeleton()
+CalSkeleton *CalModel::getSkeleton() const
 {
   return m_pSkeleton;
 }
@@ -515,11 +372,88 @@ CalSkeleton *CalModel::getSkeleton()
   *         \li \b 0 if an error happend
   *****************************************************************************/
 
-CalSpringSystem *CalModel::getSpringSystem()
+CalSpringSystem *CalModel::getSpringSystem() const
 {
   return m_pSpringSystem;
 }
 
+ /*****************************************************************************/
+/** Returns the global bounding box of the model.
+  *
+  * This function returns the global bounding box of the model.
+  *
+  * @param precision : indicate if the function need to compute a 
+  *        correct bounding box
+  *
+  * @return bounding box.
+  *****************************************************************************/
+
+
+CalBoundingBox & CalModel::getBoundingBox(bool precision)
+{
+	CalVector v;
+	v = CalVector(1.0f,0.0f,0.0f);	
+	m_boundingBox.plane[0].setNormal(v);
+	v = CalVector(-1.0f,0.0f,0.0f);	
+	m_boundingBox.plane[1].setNormal(v);
+	v = CalVector(0.0f,1.0f,0.0f);	
+	m_boundingBox.plane[2].setNormal(v);
+	v = CalVector(0.0f,-1.0f,0.0f);	
+	m_boundingBox.plane[3].setNormal(v);
+	v = CalVector(0.0f,0.0f,1.0f);	
+	m_boundingBox.plane[4].setNormal(v);
+	v = CalVector(0.0f,0.0f,-1.0f);	
+	m_boundingBox.plane[5].setNormal(v);
+
+	if(precision)
+		m_pSkeleton->calculateBoundingBoxes();
+
+	
+	std::vector<CalBone *> & vectorBone =  m_pSkeleton->getVectorBone();
+		
+	std::vector<CalBone *>::iterator iteratorBone;
+	for(iteratorBone = vectorBone.begin(); iteratorBone != vectorBone.end(); ++iteratorBone)
+	{
+		// If it's just an approximation that are needed then
+		// we just compute the bounding box from the skeleton
+
+		if(!precision || !(*iteratorBone)->getCoreBone()->isBoundingBoxPrecomputed())
+		{
+			
+			CalVector translation = (*iteratorBone)->getTranslationAbsolute();
+			
+			int planeId;
+			for(planeId = 0; planeId < 6; ++planeId)
+			{
+				if(m_boundingBox.plane[planeId].eval(translation) < 0.0f)
+				{
+					m_boundingBox.plane[planeId].setPosition(translation);
+				}
+			}
+		}
+		else
+		{
+			CalBoundingBox localBoundingBox = (*iteratorBone)->getBoundingBox();
+			CalVector points[8];
+			localBoundingBox.computePoints(points);
+			
+			for(int i=0; i < 8; i++)
+			{				
+				int planeId;
+				for(planeId = 0; planeId < 6; ++planeId)
+				{
+					if(m_boundingBox.plane[planeId].eval(points[i]) < 0.0f)
+					{
+						m_boundingBox.plane[planeId].setPosition(points[i]);
+					}
+				}
+			}				
+		}
+	}
+	
+	return m_boundingBox;
+}
+
  /*****************************************************************************/
 /** Provides access to the user data.
   *
@@ -528,7 +462,7 @@ CalSpringSystem *CalModel::getSpringSystem()
   * @return The user data stored in the model instance.
   *****************************************************************************/
 
-Cal::UserData CalModel::getUserData()
+Cal::UserData CalModel::getUserData() const
 {
   return m_userData;
 }
@@ -610,7 +544,9 @@ void CalModel::update(float deltaTime)
 {
   m_pMixer->updateAnimation(deltaTime);
   m_pMixer->updateSkeleton();
+  std::vector<CalMesh *>::iterator iteratorVectorMesh = m_vectorMesh.begin();
   // m_pMorpher->update(...);
+  m_pMorphTargetMixer->update(deltaTime);
   m_pPhysique->update();
   m_pSpringSystem->update(deltaTime);
 }
