@@ -16,15 +16,16 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "error.h"
-#include "physique.h"
-#include "coremodel.h"
-#include "model.h"
-#include "mesh.h"
-#include "submesh.h"
-#include "skeleton.h"
-#include "bone.h"
-#include "coresubmesh.h"
+#include "cal3d/error.h"
+#include "cal3d/physique.h"
+#include "cal3d/coremodel.h"
+#include "cal3d/model.h"
+#include "cal3d/mesh.h"
+#include "cal3d/submesh.h"
+#include "cal3d/skeleton.h"
+#include "cal3d/bone.h"
+#include "cal3d/coresubmesh.h"
+#include "cal3d/coresubmorphtarget.h"
 
  /*****************************************************************************/
 /** Constructs the physique instance.
@@ -32,20 +33,12 @@
   * This function is the default constructor of the physique instance.
   *****************************************************************************/
 
-CalPhysique::CalPhysique()
-{
-  m_pModel = 0;
-  m_Normalize = true;
-}
-
- /*****************************************************************************/
-/** Destructs the physique instance.
-  *
-  * This function is the destructor of the physique instance.
-  *****************************************************************************/
-
-CalPhysique::~CalPhysique()
+CalPhysique::CalPhysique(CalModel* pModel)
+  : m_pModel(0)
+  , m_Normalize(true)
 {
+  assert(pModel);
+  m_pModel = pModel;
 }
 
  /*****************************************************************************/
@@ -62,8 +55,13 @@ CalPhysique::~CalPhysique()
   * @return The number of vertices written to the buffer.
   *****************************************************************************/
 
-int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
+int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer, int stride)
 {
+  if(stride <= 0)
+  {
+	  stride = 3*sizeof(float);
+  }
+
   // get bone vector of the skeleton
   std::vector<CalBone *>& vectorBone = m_pModel->getSkeleton()->getVectorBone();
 
@@ -76,6 +74,16 @@ int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
   // get the number of vertices
   int vertexCount;
   vertexCount = pSubmesh->getVertexCount();
+  
+  // get the sub morph target vector from the core sub mesh
+  std::vector<CalCoreSubMorphTarget*>& vectorSubMorphTarget =
+  pSubmesh->getCoreSubmesh()->getVectorCoreSubMorphTarget();
+
+  // calculate the base weight
+  float baseWeight = pSubmesh->getBaseWeight();
+
+  // get the number of morph targets
+  int morphTargetCount = pSubmesh->getMorphTargetWeightCount();
 
   // calculate all submesh vertices
   int vertexId;
@@ -83,6 +91,31 @@ int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
   {
     // get the vertex
     CalCoreSubmesh::Vertex& vertex = vectorVertex[vertexId];
+    
+    // blend the morph targets
+    CalVector position(0,0,0);
+    if(baseWeight == 1.0f)
+    {
+       position.x = vertex.position.x;
+       position.y = vertex.position.y;
+       position.z = vertex.position.z;
+    }
+    else
+    {
+      position.x = baseWeight*vertex.position.x;
+      position.y = baseWeight*vertex.position.y;
+      position.z = baseWeight*vertex.position.z;
+      int morphTargetId;
+      for(morphTargetId=0; morphTargetId < morphTargetCount;++morphTargetId)
+      {
+        CalCoreSubMorphTarget::BlendVertex& blendVertex =
+        vectorSubMorphTarget[morphTargetId]->getVectorBlendVertex()[vertexId];
+        float currentWeight = pSubmesh->getMorphTargetWeight(morphTargetId);
+        position.x += currentWeight*blendVertex.position.x;
+        position.y += currentWeight*blendVertex.position.y;
+        position.z += currentWeight*blendVertex.position.z;
+      }
+    }
 
     // initialize vertex
     float x, y, z;
@@ -91,26 +124,34 @@ int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
     z = 0.0f;
 
     // blend together all vertex influences
-    int influenceId;
-	int influenceCount=(int)vertex.vectorInfluence.size();
-    for(influenceId = 0; influenceId < influenceCount; ++influenceId)
-    {
-      // get the influence
-      CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
-
-      // get the bone of the influence vertex
-      CalBone *pBone;
-      pBone = vectorBone[influence.boneId];
-
-      // transform vertex with current state of the bone
-      CalVector v(vertex.position);
-      v *= pBone->getTransformMatrix();
-      v += pBone->getTranslationBoneSpace();
-
-      x += influence.weight * v.x;
-      y += influence.weight * v.y;
-      z += influence.weight * v.z;
-    }
+    size_t influenceCount=vertex.vectorInfluence.size();
+    if(influenceCount == 0) 
+	{
+      x = position.x;
+      y = position.y;
+      z = position.z;
+    } 
+	else 
+	{
+		for(size_t influenceId = 0; influenceId < influenceCount; ++influenceId)
+		{
+			// get the influence
+			CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
+			
+			// get the bone of the influence vertex
+			CalBone *pBone;
+			pBone = vectorBone[influence.boneId];
+			
+			// transform vertex with current state of the bone
+			CalVector v(position);
+			v *= pBone->getTransformMatrix();
+			v += pBone->getTranslationBoneSpace();
+			
+			x += influence.weight * v.x;
+			y += influence.weight * v.y;
+			z += influence.weight * v.z;
+		}
+	}
 
     // save vertex position
     if(pSubmesh->getCoreSubmesh()->getSpringCount() > 0 && pSubmesh->hasInternalData())
@@ -134,12 +175,135 @@ int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
     }
 
     // next vertex position in buffer
-    pVertexBuffer += 3;
+    pVertexBuffer = (float *)(((char *)pVertexBuffer) + stride) ;
   }
 
   return vertexCount;
 }
 
+ /*****************************************************************************/
+/** Calculates one transformed vertex.
+  *
+  * This function calculates and returns a transformed vertex of a
+  * specific submesh.
+  *
+  * @param pSubmesh A pointer to the submesh from which the vertex should
+  *                 be calculated and returned.
+  * @param vertexId The id of the vertex that should be transformed.
+  *
+  * @return The number of vertices written to the buffer.
+  *****************************************************************************/
+
+CalVector CalPhysique::calculateVertex(CalSubmesh *pSubmesh, int vertexId)
+{
+  // get bone vector of the skeleton
+  std::vector<CalBone *>& vectorBone = m_pModel->getSkeleton()->getVectorBone();
+
+  // get vertex of the core submesh
+  std::vector<CalCoreSubmesh::Vertex>& vectorVertex = pSubmesh->getCoreSubmesh()->getVectorVertex();
+
+  // get physical property vector of the core submesh
+  //std::vector<CalCoreSubmesh::PhysicalProperty>& vectorPhysicalProperty = pSubmesh->getCoreSubmesh()->getVectorPhysicalProperty();
+
+  // get the sub morph target vector from the core sub mesh
+  std::vector<CalCoreSubMorphTarget*>& vectorSubMorphTarget =
+  pSubmesh->getCoreSubmesh()->getVectorCoreSubMorphTarget();
+
+  // calculate the base weight
+  float baseWeight = pSubmesh->getBaseWeight();
+
+  // get the number of morph targets
+  int morphTargetCount = pSubmesh->getMorphTargetWeightCount();
+
+  // get the vertex
+  CalCoreSubmesh::Vertex& vertex = vectorVertex[vertexId];
+
+  // blend the morph targets
+  CalVector position(0,0,0);
+  if(baseWeight == 1.0f)
+  {
+    position.x = vertex.position.x;
+    position.y = vertex.position.y;
+    position.z = vertex.position.z;
+  }
+  else
+  {
+    position.x = baseWeight*vertex.position.x;
+    position.y = baseWeight*vertex.position.y;
+    position.z = baseWeight*vertex.position.z;
+    int morphTargetId;
+    for(morphTargetId=0; morphTargetId < morphTargetCount;++morphTargetId)
+    {
+      CalCoreSubMorphTarget::BlendVertex& blendVertex =
+	vectorSubMorphTarget[morphTargetId]->getVectorBlendVertex()[vertexId];
+      float currentWeight = pSubmesh->getMorphTargetWeight(morphTargetId);
+      position.x += currentWeight*blendVertex.position.x;
+      position.y += currentWeight*blendVertex.position.y;
+      position.z += currentWeight*blendVertex.position.z;
+    }
+  }
+
+  // initialize vertex
+  float x, y, z;
+  x = 0.0f;
+  y = 0.0f;
+  z = 0.0f;
+
+  // blend together all vertex influences
+  int influenceId;
+  int influenceCount=(int)vertex.vectorInfluence.size();
+  if(influenceCount == 0) 
+  {
+    x = position.x;
+    y = position.y;
+    z = position.z;
+  } 
+  else 
+  {
+	  for(influenceId = 0; influenceId < influenceCount; ++influenceId)
+	  {
+		  // get the influence
+		  CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
+		  
+		  // get the bone of the influence vertex
+		  CalBone *pBone;
+		  pBone = vectorBone[influence.boneId];
+		  
+		  // transform vertex with current state of the bone
+		  CalVector v(position);
+		  v *= pBone->getTransformMatrix();
+		  v += pBone->getTranslationBoneSpace();
+		  
+		  x += influence.weight * v.x;
+		  y += influence.weight * v.y;
+		  z += influence.weight * v.z;
+	  }
+  }
+  /*
+  // save vertex position
+  if(pSubmesh->getCoreSubmesh()->getSpringCount() > 0 && pSubmesh->hasInternalData())
+  {
+    // get the pgysical property of the vertex
+    CalCoreSubmesh::PhysicalProperty& physicalProperty = vectorPhysicalProperty[vertexId];
+
+    // assign new vertex position if there is no vertex weight
+    if(physicalProperty.weight == 0.0f)
+    {
+      pVertexBuffer[0] = x;
+      pVertexBuffer[1] = y;
+      pVertexBuffer[2] = z;
+    }
+  }
+  else
+  {
+    pVertexBuffer[0] = x;
+    pVertexBuffer[1] = y;
+    pVertexBuffer[2] = z;
+  }
+  */
+  // return the vertex
+  return CalVector(x,y,z);
+}
  /*****************************************************************************/
 /** Calculates the transformed tangent space data.
   *
@@ -155,9 +319,13 @@ int CalPhysique::calculateVertices(CalSubmesh *pSubmesh, float *pVertexBuffer)
   * @return The number of tangent spaces written to the buffer.
   *****************************************************************************/
 
-int CalPhysique::calculateTangentSpaces(CalSubmesh *pSubmesh, int mapId, float *pTangentSpaceBuffer)
+int CalPhysique::calculateTangentSpaces(CalSubmesh *pSubmesh, int mapId, float *pTangentSpaceBuffer, int stride)
 {
   if((mapId < 0) || (mapId >= (int)pSubmesh->getCoreSubmesh()->getVectorVectorTangentSpace().size())) return false;
+  if(stride <= 0)
+  {
+	  stride = 4*sizeof(float);
+  }
 
   // get bone vector of the skeleton
   std::vector<CalBone *>& vectorBone = m_pModel->getSkeleton()->getVectorBone();
@@ -189,7 +357,8 @@ int CalPhysique::calculateTangentSpaces(CalSubmesh *pSubmesh, int mapId, float *
 
     // blend together all vertex influences
     int influenceId;
-	int influenceCount=(int)vertex.vectorInfluence.size();
+    int influenceCount=(int)vertex.vectorInfluence.size();
+
     for(influenceId = 0; influenceId < influenceCount; influenceId++)
     {
       // get the influence
@@ -227,7 +396,7 @@ int CalPhysique::calculateTangentSpaces(CalSubmesh *pSubmesh, int mapId, float *
 
     pTangentSpaceBuffer[3] = tangentSpace.crossFactor;
     // next vertex position in buffer
-    pTangentSpaceBuffer += 4;
+    pTangentSpaceBuffer = (float *)(((char *)pTangentSpaceBuffer) + stride) ;
   }
 
   return vertexCount;
@@ -248,8 +417,13 @@ int CalPhysique::calculateTangentSpaces(CalSubmesh *pSubmesh, int mapId, float *
   * @return The number of normals written to the buffer.
   *****************************************************************************/
 
-int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
+int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer, int stride)
 {
+  if(stride <= 0)
+  {
+	  stride = 3*sizeof(float);
+  }
+
   // get bone vector of the skeleton
   std::vector<CalBone *>& vectorBone = m_pModel->getSkeleton()->getVectorBone();
 
@@ -260,6 +434,16 @@ int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
   int vertexCount;
   vertexCount = pSubmesh->getVertexCount();
 
+  // get the sub morph target vector from the core sub mesh
+  std::vector<CalCoreSubMorphTarget*>& vectorSubMorphTarget =
+  pSubmesh->getCoreSubmesh()->getVectorCoreSubMorphTarget();
+
+  // calculate the base weight
+  float baseWeight = pSubmesh->getBaseWeight();
+
+  // get the number of morph targets
+  int morphTargetCount = pSubmesh->getMorphTargetWeightCount();
+
   // calculate normal for all submesh vertices
   int vertexId;
   for(vertexId = 0; vertexId < vertexCount; ++vertexId)
@@ -267,6 +451,31 @@ int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
     // get the vertex
     CalCoreSubmesh::Vertex& vertex = vectorVertex[vertexId];
 
+    // blend the morph targets
+    CalVector normal(0,0,0);
+    if(baseWeight == 1.0f)
+    {
+      normal.x = vertex.normal.x;
+      normal.y = vertex.normal.y;
+      normal.z = vertex.normal.z;
+    }
+    else
+    {
+      normal.x = baseWeight*vertex.normal.x;
+      normal.y = baseWeight*vertex.normal.y;
+      normal.z = baseWeight*vertex.normal.z;
+      int morphTargetId;
+      for(morphTargetId=0; morphTargetId < morphTargetCount;++morphTargetId)
+      {
+        CalCoreSubMorphTarget::BlendVertex& blendVertex = 
+        vectorSubMorphTarget[morphTargetId]->getVectorBlendVertex()[vertexId];
+        float currentWeight = pSubmesh->getMorphTargetWeight(morphTargetId);
+        normal.x += currentWeight*blendVertex.normal.x;
+        normal.y += currentWeight*blendVertex.normal.y;
+        normal.z += currentWeight*blendVertex.normal.z;
+      }
+    }
+
     // initialize normal
     float nx, ny, nz;
     nx = 0.0f;
@@ -276,23 +485,32 @@ int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
     // blend together all vertex influences
     int influenceId;
 	int influenceCount=(int)vertex.vectorInfluence.size();
-    for(influenceId = 0; influenceId < influenceCount; ++influenceId)
-    {
-      // get the influence
-      CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
-
-      // get the bone of the influence vertex
-      CalBone *pBone;
-      pBone = vectorBone[influence.boneId];
-
-      // transform normal with current state of the bone
-      CalVector v(vertex.normal);
-      v *= pBone->getTransformMatrix(); 
-
-      nx += influence.weight * v.x;
-      ny += influence.weight * v.y;
-      nz += influence.weight * v.z;
-    }
+    if(influenceCount == 0) 
+	{
+      nx = normal.x;
+      ny = normal.y;
+      nz = normal.z;
+    } 
+	else 
+	{
+		for(influenceId = 0; influenceId < influenceCount; ++influenceId)
+		{
+			// get the influence
+			CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
+			
+			// get the bone of the influence vertex
+			CalBone *pBone;
+			pBone = vectorBone[influence.boneId];
+			
+			// transform normal with current state of the bone
+			CalVector v(normal);
+			v *= pBone->getTransformMatrix(); 
+			
+			nx += influence.weight * v.x;
+			ny += influence.weight * v.y;
+			nz += influence.weight * v.z;
+		}
+	}
 
     // re-normalize normal if necessary
     if (m_Normalize)
@@ -312,7 +530,7 @@ int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
     } 
 
     // next vertex position in buffer
-    pNormalBuffer += 3;
+    pNormalBuffer = (float *)(((char *)pNormalBuffer) + stride) ;
   }
 
   return vertexCount;
@@ -332,8 +550,13 @@ int CalPhysique::calculateNormals(CalSubmesh *pSubmesh, float *pNormalBuffer)
   * @return The number of vertices written to the buffer.
   *****************************************************************************/
 
-int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVertexBuffer)
+int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVertexBuffer, int stride)
 {
+  if(stride <= 0)
+  {
+	  stride = 6*sizeof(float);
+  }
+
   // get bone vector of the skeleton
   std::vector<CalBone *>& vectorBone = m_pModel->getSkeleton()->getVectorBone();
 
@@ -347,6 +570,16 @@ int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVerte
   int vertexCount;
   vertexCount = pSubmesh->getVertexCount();
 
+  // get the sub morph target vector from the core sub mesh
+  std::vector<CalCoreSubMorphTarget*>& vectorSubMorphTarget =
+  pSubmesh->getCoreSubmesh()->getVectorCoreSubMorphTarget();
+
+  // calculate the base weight
+  float baseWeight = pSubmesh->getBaseWeight();
+
+  // get the number of morph targets
+  int morphTargetCount = pSubmesh->getMorphTargetWeightCount();
+
   // calculate all submesh vertices
   int vertexId;
   for(vertexId = 0; vertexId < vertexCount; ++vertexId)
@@ -354,6 +587,41 @@ int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVerte
     // get the vertex
     CalCoreSubmesh::Vertex& vertex = vectorVertex[vertexId];
 
+    // blend the morph targets
+    CalVector position(0,0,0);
+    CalVector normal(0,0,0);
+    if(baseWeight == 1.0f)
+    {
+      position.x = vertex.position.x;
+      position.y = vertex.position.y;
+      position.z = vertex.position.z;
+      normal.x = vertex.normal.x;
+      normal.y = vertex.normal.y;
+      normal.z = vertex.normal.z;
+    }
+    else
+    {
+      position.x = baseWeight*vertex.position.x;
+      position.y = baseWeight*vertex.position.y;
+      position.z = baseWeight*vertex.position.z;
+      normal.x = baseWeight*vertex.normal.x;
+      normal.y = baseWeight*vertex.normal.y;
+      normal.z = baseWeight*vertex.normal.z;
+      int morphTargetId;
+      for(morphTargetId=0; morphTargetId < morphTargetCount;++morphTargetId)
+      {
+        CalCoreSubMorphTarget::BlendVertex& blendVertex =
+        vectorSubMorphTarget[morphTargetId]->getVectorBlendVertex()[vertexId];
+        float currentWeight = pSubmesh->getMorphTargetWeight(morphTargetId);
+        position.x += currentWeight*blendVertex.position.x;
+        position.y += currentWeight*blendVertex.position.y;
+        position.z += currentWeight*blendVertex.position.z;
+        normal.x += currentWeight*blendVertex.normal.x;
+        normal.y += currentWeight*blendVertex.normal.y;
+        normal.z += currentWeight*blendVertex.normal.z;
+      }
+    }
+
     // initialize vertex
     float x, y, z;
     x = 0.0f;
@@ -369,32 +637,44 @@ int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVerte
     // blend together all vertex influences
     int influenceId;
 	int influenceCount=(int)vertex.vectorInfluence.size();
-    for(influenceId = 0; influenceId < influenceCount; ++influenceId)
-    {
-      // get the influence
-      CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
-
-      // get the bone of the influence vertex
-      CalBone *pBone;
-      pBone = vectorBone[influence.boneId];
-
-      // transform vertex with current state of the bone
-      CalVector v(vertex.position);
-      v *= pBone->getTransformMatrix();
-      v += pBone->getTranslationBoneSpace();
-
-      x += influence.weight * v.x;
-      y += influence.weight * v.y;
-      z += influence.weight * v.z;
-
-	  // transform normal with current state of the bone
-      CalVector n(vertex.normal);
-      n *= pBone->getTransformMatrix();
-
-      nx += influence.weight * n.x;
-      ny += influence.weight * n.y;
-      nz += influence.weight * n.z;
-    }
+    if(influenceCount == 0) 
+	{
+      x = position.x;
+      y = position.y;
+      z = position.z;
+      nx = normal.x;
+      ny = normal.y;
+      nz = normal.z;
+    } 
+	else 
+	{
+		for(influenceId = 0; influenceId < influenceCount; ++influenceId)
+		{
+			// get the influence
+			CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
+			
+			// get the bone of the influence vertex
+			CalBone *pBone;
+			pBone = vectorBone[influence.boneId];
+			
+			// transform vertex with current state of the bone
+			CalVector v(position);
+			v *= pBone->getTransformMatrix();
+			v += pBone->getTranslationBoneSpace();
+			
+			x += influence.weight * v.x;
+			y += influence.weight * v.y;
+			z += influence.weight * v.z;
+			
+			// transform normal with current state of the bone
+			CalVector n(normal);
+			n *= pBone->getTransformMatrix();
+			
+			nx += influence.weight * n.x;
+			ny += influence.weight * n.y;
+			nz += influence.weight * n.z;
+		}
+	}
 
     // save vertex position
     if(pSubmesh->getCoreSubmesh()->getSpringCount() > 0 && pSubmesh->hasInternalData())
@@ -436,8 +716,7 @@ int CalPhysique::calculateVerticesAndNormals(CalSubmesh *pSubmesh, float *pVerte
 
 
 	// next vertex position in buffer	
-	pVertexBuffer += 6;
-
+    pVertexBuffer = (float *)(((char *)pVertexBuffer) + stride) ;
   }
 
   return vertexCount;
@@ -491,6 +770,16 @@ int CalPhysique::calculateVerticesNormalsAndTexCoords(CalSubmesh *pSubmesh, floa
   int vertexCount;
   vertexCount = pSubmesh->getVertexCount();
 
+  // get the sub morph target vector from the core sub mesh
+  std::vector<CalCoreSubMorphTarget*>& vectorSubMorphTarget =
+  pSubmesh->getCoreSubmesh()->getVectorCoreSubMorphTarget();
+
+  // calculate the base weight
+  float baseWeight = pSubmesh->getBaseWeight();
+
+  // get the number of morph targets
+  int morphTargetCount = pSubmesh->getMorphTargetWeightCount();
+
   // calculate all submesh vertices
   int vertexId;
   for(vertexId = 0; vertexId < vertexCount; ++vertexId)
@@ -498,6 +787,41 @@ int CalPhysique::calculateVerticesNormalsAndTexCoords(CalSubmesh *pSubmesh, floa
     // get the vertex
     CalCoreSubmesh::Vertex& vertex = vectorVertex[vertexId];
 
+    // blend the morph targets
+    CalVector position(0,0,0);
+    CalVector normal(0,0,0);
+    if(baseWeight == 1.0f)
+    {
+      position.x = vertex.position.x;
+      position.y = vertex.position.y;
+      position.z = vertex.position.z;
+      normal.x = vertex.normal.x;
+      normal.y = vertex.normal.y;
+      normal.z = vertex.normal.z;
+    }
+    else
+    {
+      position.x = baseWeight*vertex.position.x;
+      position.y = baseWeight*vertex.position.y;
+      position.z = baseWeight*vertex.position.z;
+      normal.x = baseWeight*vertex.normal.x;
+      normal.y = baseWeight*vertex.normal.y;
+      normal.z = baseWeight*vertex.normal.z;
+      int morphTargetId;
+      for(morphTargetId=0; morphTargetId < morphTargetCount;++morphTargetId)
+      {
+        CalCoreSubMorphTarget::BlendVertex& blendVertex =
+        vectorSubMorphTarget[morphTargetId]->getVectorBlendVertex()[vertexId];
+        float currentWeight = pSubmesh->getMorphTargetWeight(morphTargetId);
+        position.x += currentWeight*blendVertex.position.x;
+        position.y += currentWeight*blendVertex.position.y;
+        position.z += currentWeight*blendVertex.position.z;
+        normal.x += currentWeight*blendVertex.normal.x;
+        normal.y += currentWeight*blendVertex.normal.y;
+        normal.z += currentWeight*blendVertex.normal.z;
+      }
+    }
+
     // initialize vertex
     float x, y, z;
     x = 0.0f;
@@ -513,32 +837,44 @@ int CalPhysique::calculateVerticesNormalsAndTexCoords(CalSubmesh *pSubmesh, floa
     // blend together all vertex influences
     int influenceId;
 	int influenceCount=(int)vertex.vectorInfluence.size();
-    for(influenceId = 0; influenceId < influenceCount; ++influenceId)
-    {
-      // get the influence
-      CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
-
-      // get the bone of the influence vertex
-      CalBone *pBone;
-      pBone = vectorBone[influence.boneId];
-
-      // transform vertex with current state of the bone
-      CalVector v(vertex.position);
-      v *= pBone->getTransformMatrix();
-      v += pBone->getTranslationBoneSpace();
-
-      x += influence.weight * v.x;
-      y += influence.weight * v.y;
-      z += influence.weight * v.z;
-
-	  // transform normal with current state of the bone
-      CalVector n(vertex.normal);	  
-      n *= pBone->getTransformMatrix();
-
-      nx += influence.weight * n.x;
-      ny += influence.weight * n.y;
-      nz += influence.weight * n.z;
-    }
+	if(influenceCount == 0) 
+	{
+      x = position.x;
+      y = position.y;
+      z = position.z;
+      nx = normal.x;
+      ny = normal.y;
+      nz = normal.z;
+    } 
+	else 
+	{
+		for(influenceId = 0; influenceId < influenceCount; ++influenceId)
+		{
+			// get the influence
+			CalCoreSubmesh::Influence& influence = vertex.vectorInfluence[influenceId];
+			
+			// get the bone of the influence vertex
+			CalBone *pBone;
+			pBone = vectorBone[influence.boneId];
+			
+			// transform vertex with current state of the bone
+			CalVector v(position);
+			v *= pBone->getTransformMatrix();
+			v += pBone->getTranslationBoneSpace();
+			
+			x += influence.weight * v.x;
+			y += influence.weight * v.y;
+			z += influence.weight * v.z;
+			
+			// transform normal with current state of the bone
+			CalVector n(normal);	  
+			n *= pBone->getTransformMatrix();
+			
+			nx += influence.weight * n.x;
+			ny += influence.weight * n.y;
+			nz += influence.weight * n.z;
+		}
+	}
 
     // save vertex position
     if(pSubmesh->getCoreSubmesh()->getSpringCount() > 0 && pSubmesh->hasInternalData())
@@ -599,53 +935,6 @@ int CalPhysique::calculateVerticesNormalsAndTexCoords(CalSubmesh *pSubmesh, floa
 }
 
 
-
- /*****************************************************************************/
-/** Creates the physique instance.
-  *
-  * This function creates the physique instance.
-  *
-  * @param pModel A pointer to the model that should be managed with this
-  *               physique instance.
-  *
-  * @return One of the following values:
-  *         \li \b true if successful
-  *         \li \b false if an error happend
-  *****************************************************************************/
-
-bool CalPhysique::create(CalModel *pModel)
-{
-  if(pModel == 0)
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
-  m_pModel = pModel;
-
-  CalCoreModel *pCoreModel;
-  pCoreModel = m_pModel->getCoreModel();
-  if(pCoreModel == 0)
-  {
-    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return false;
-  }
-
-  return true;
-}
-
- /*****************************************************************************/
-/** Destroys the physique instance.
-  *
-  * This function destroys all data stored in the physique instance and frees
-  * all allocated memory.
-  *****************************************************************************/
-
-void CalPhysique::destroy()
-{
-  m_pModel = 0;
-}
-
  /*****************************************************************************/
 /** Updates all the internally handled attached meshes.
   *
@@ -680,15 +969,16 @@ void CalPhysique::update()
         std::vector<CalVector>& vectorNormal = (*iteratorSubmesh)->getVectorNormal();
         calculateNormals(*iteratorSubmesh, (float *)&vectorNormal[0]);
 
-		unsigned mapId;
-		for(mapId=0;mapId< (*iteratorSubmesh)->getVectorVectorTangentSpace().size();mapId++)
-		{
-			if((*iteratorSubmesh)->isTangentsEnabled(mapId))
-			{
-                std::vector<CalSubmesh::TangentSpace>& vectorTangentSpace = (*iteratorSubmesh)->getVectorVectorTangentSpace()[mapId];
-                calculateTangentSpaces(*iteratorSubmesh, mapId,(float *)&vectorTangentSpace[0]);
-			}
-		}
+        unsigned mapId;
+        for(mapId=0;mapId< (*iteratorSubmesh)->getVectorVectorTangentSpace().size();mapId++)
+        {
+          if((*iteratorSubmesh)->isTangentsEnabled(mapId))
+          {
+            std::vector<CalSubmesh::TangentSpace>& vectorTangentSpace = (*iteratorSubmesh)->getVectorVectorTangentSpace()[mapId];
+            calculateTangentSpaces(*iteratorSubmesh, mapId,(float *)&vectorTangentSpace[0]);
+          }
+        }
+
       }
     }
   }
