@@ -16,42 +16,20 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "saver.h"
-#include "error.h"
-#include "vector.h"
-#include "quaternion.h"
-#include "coremodel.h"
-#include "coreskeleton.h"
-#include "corebone.h"
-#include "coreanimation.h"
-#include "coretrack.h"
-#include "corekeyframe.h"
-#include "coremesh.h"
-#include "coresubmesh.h"
-#include "corematerial.h"
-
-#ifdef CAL_USE_XML
-#include "tinyxml.h"
-#endif
- /*****************************************************************************/
-/** Constructs the saver instance.
-  *
-  * This function is the default constructor of the saver instance.
-  *****************************************************************************/
-
-CalSaver::CalSaver()
-{
-}
-
- /*****************************************************************************/
-/** Destructs the saver instance.
-  *
-  * This function is the destructor of the saver instance.
-  *****************************************************************************/
-
-CalSaver::~CalSaver()
-{
-}
+#include "cal3d/saver.h"
+#include "cal3d/error.h"
+#include "cal3d/vector.h"
+#include "cal3d/quaternion.h"
+#include "cal3d/coremodel.h"
+#include "cal3d/coreskeleton.h"
+#include "cal3d/corebone.h"
+#include "cal3d/coreanimation.h"
+#include "cal3d/coretrack.h"
+#include "cal3d/corekeyframe.h"
+#include "cal3d/coremesh.h"
+#include "cal3d/coresubmesh.h"
+#include "cal3d/corematerial.h"
+#include "cal3d/tinyxml.h"
 
  /*****************************************************************************/
 /** Saves a core animation instance.
@@ -70,10 +48,8 @@ CalSaver::~CalSaver()
 
 bool CalSaver::saveCoreAnimation(const std::string& strFilename, CalCoreAnimation *pCoreAnimation)
 {
-#ifdef CAL_USE_XML
   if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::ANIMATION_XMLFILE_MAGIC)==0)
 	 return saveXmlCoreAnimation(strFilename, pCoreAnimation);	
-#endif
 
   // open the file
   std::ofstream file;
@@ -129,6 +105,8 @@ bool CalSaver::saveCoreAnimation(const std::string& strFilename, CalCoreAnimatio
   // explicitly close the file
   file.close();
 
+  pCoreAnimation->setFilename(strFilename);
+
   return true;
 }
 
@@ -285,10 +263,8 @@ bool CalSaver::saveCoreKeyframe(std::ofstream& file, const std::string& strFilen
 
 bool CalSaver::saveCoreMaterial(const std::string& strFilename, CalCoreMaterial *pCoreMaterial)
 {
-#ifdef CAL_USE_XML
-	if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::MATERIAL_XMLFILE_MAGIC)==0)
-	 return saveXmlCoreMaterial(strFilename, pCoreMaterial);	
-#endif
+  if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::MATERIAL_XMLFILE_MAGIC)==0)
+    return saveXmlCoreMaterial(strFilename, pCoreMaterial);	
 
   // open the file
   std::ofstream file;
@@ -365,6 +341,8 @@ bool CalSaver::saveCoreMaterial(const std::string& strFilename, CalCoreMaterial
   // explicitly close the file
   file.close();
 
+  pCoreMaterial->setFilename(strFilename);
+
   return true;
 }
 
@@ -383,10 +361,8 @@ bool CalSaver::saveCoreMaterial(const std::string& strFilename, CalCoreMaterial
 
 bool CalSaver::saveCoreMesh(const std::string& strFilename, CalCoreMesh *pCoreMesh)
 {
-#ifdef CAL_USE_XML
   if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::MESH_XMLFILE_MAGIC)==0)
-	 return saveXmlCoreMesh(strFilename, pCoreMesh);
-#endif
+    return saveXmlCoreMesh(strFilename, pCoreMesh);
 
   // open the file
   std::ofstream file;
@@ -435,6 +411,8 @@ bool CalSaver::saveCoreMesh(const std::string& strFilename, CalCoreMesh *pCoreMe
   // explicitly close the file
   file.close();
 
+  pCoreMesh->setFilename(strFilename);
+
   return true;
 }
 
@@ -455,10 +433,8 @@ bool CalSaver::saveCoreMesh(const std::string& strFilename, CalCoreMesh *pCoreMe
 
 bool CalSaver::saveCoreSkeleton(const std::string& strFilename, CalCoreSkeleton *pCoreSkeleton)
 {
-#ifdef CAL_USE_XML
-	if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::SKELETON_XMLFILE_MAGIC)==0)
-	 return saveXmlCoreSkeleton(strFilename, pCoreSkeleton);	
-#endif
+  if(strFilename.size()>= 3 && stricmp(strFilename.substr(strFilename.size()-3,3).c_str(),Cal::SKELETON_XMLFILE_MAGIC)==0)
+    return saveXmlCoreSkeleton(strFilename, pCoreSkeleton);	
 
   // open the file
   std::ofstream file;
@@ -730,7 +706,6 @@ bool CalSaver::saveCoreTrack(std::ofstream& file, const std::string& strFilename
   return true;
 }
 
-#ifdef CAL_USE_XML
  /*****************************************************************************/
 /** Saves a core skeleton instance to a XML file
   *
@@ -751,14 +726,12 @@ bool CalSaver::saveXmlCoreSkeleton(const std::string& strFilename, CalCoreSkelet
   std::stringstream str;
 
   TiXmlDocument doc(strFilename);  
-  
-  TiXmlElement header("HEADER");
-  header.SetAttribute("MAGIC",Cal::SKELETON_XMLFILE_MAGIC);
-  header.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
-  
-  doc.InsertEndChild(header);
+
 
   TiXmlElement skeleton("SKELETON");
+  //skeleton.SetAttribute("MAGIC",Cal::SKELETON_XMLFILE_MAGIC);
+  skeleton.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
+
   skeleton.SetAttribute("NUMBONES",pCoreSkeleton->getVectorCoreBone().size());
   
 
@@ -889,14 +862,11 @@ bool CalSaver::saveXmlCoreAnimation(const std::string& strFilename, CalCoreAnima
     std::stringstream str;
 
     TiXmlDocument doc(strFilename);
-  
+
     TiXmlElement animation("ANIMATION");
+    //animation.SetAttribute("MAGIC",Cal::ANIMATION_XMLFILE_MAGIC);
+    animation.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
 
-	TiXmlElement header("HEADER");
-	header.SetAttribute("MAGIC",Cal::ANIMATION_XMLFILE_MAGIC);
-	header.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
-
-	doc.InsertEndChild(header);
 
 	str.str("");
     str << pCoreAnimation->getDuration();	
@@ -994,13 +964,10 @@ bool CalSaver::saveXmlCoreMesh(const std::string& strFilename, CalCoreMesh *pCor
 
     TiXmlDocument doc(strFilename);    
 	
-	TiXmlElement header("HEADER");
-	header.SetAttribute("MAGIC",Cal::MESH_XMLFILE_MAGIC);
-	header.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
-
-    doc.InsertEndChild(header);
 
 	TiXmlElement mesh("MESH");
+	//mesh.SetAttribute("MAGIC",Cal::MESH_XMLFILE_MAGIC);
+	mesh.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
 	mesh.SetAttribute("NUMSUBMESH",pCoreMesh->getCoreSubmeshCount());
 
 	// get the submesh vector
@@ -1219,16 +1186,11 @@ bool CalSaver::saveXmlCoreMaterial(const std::string& strFilename, CalCoreMateri
   std::stringstream str;
 
   TiXmlDocument doc(strFilename);
-  
-  
-  
-  TiXmlElement header("HEADER");
-  header.SetAttribute("MAGIC",Cal::MATERIAL_XMLFILE_MAGIC);
-  header.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
-
-  doc.InsertEndChild(header);
 
   TiXmlElement material("MATERIAL");
+  //material.SetAttribute("MAGIC",Cal::MATERIAL_XMLFILE_MAGIC);
+  material.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
+
   material.SetAttribute("NUMMAPS",pCoreMaterial->getVectorMap().size());
   
   TiXmlElement ambient("AMBIENT");
@@ -1314,5 +1276,4 @@ bool CalSaver::saveXmlCoreMaterial(const std::string& strFilename, CalCoreMateri
   return true;
 
 }
-#endif
 //****************************************************************************//
